"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function isGen(fn) {
    return (typeof fn == 'function' && fn.constructor.name == 'GeneratorFunction');
}
function createDeferred() {
    let r;
    let j;
    const promise = new Promise((resolve, reject) => {
        r = resolve;
        j = reject;
    });
    if (!r || !j) {
        throw new Error('Creating Deferred failed');
    }
    return { promise, resolve: r, reject: j };
}
function generatorToPromise(generatorFunction) {
    if (!isGen(generatorFunction)) {
        throw new Error('The given function must be a generator function');
    }
    return function invoker() {
        const deferred = createDeferred();
        const generator = generatorFunction.call(this, arguments);
        (function next(error, value) {
            let genState = null;
            try {
                if (error)
                    genState = generator.throw(error);
                else
                    genState = generator.next(value);
            }
            catch (e) {
                genState = { value: Promise.reject(e), done: true };
            }
            if (genState.done) {
                deferred.resolve(genState.value);
            }
            else {
                Promise.resolve(genState.value)
                    .then(promiseResult => next(null, promiseResult))
                    .catch(error => next(error));
            }
        })();
        return deferred.promise;
    };
}
exports.default = generatorToPromise;
//# sourceMappingURL=generator-to-promise.js.map